'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var reactRedux = require('react-redux');

function createAction(prefix, actionType) {
  var type = prefix + '/' + actionType;

  function action(payload) {
    return { type: type, payload: payload };
  }

  action.type = type;

  return action;
}

function createActions(prefix) {
  return {
    setState: createAction(prefix, 'set-state'),
    remove: createAction(prefix, 'remove')
  };
}

function createReducer(options) {
  var _createActions = createActions(options.prefix),
      setState = _createActions.setState,
      remove = _createActions.remove;

  return function reducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];

    if (action.type !== setState.type && action.type !== remove.type) {
      return state;
    }

    var _action$payload = action.payload,
        name = _action$payload.name,
        id = _action$payload.id,
        componentState = _action$payload.state;


    var nextState = Object.assign({}, state);
    var tree = Object.assign({}, state[name]);

    nextState[name] = tree;

    if (action.type === setState.type) {
      tree[id] = Object.assign({}, tree[id], componentState);
    }

    if (action.type === remove.type) {
      delete tree[id];
      if (Object.keys(tree).length === 0) {
        delete nextState[name];
      }
    }

    return nextState;
  };
}

function getDisplayName(Component) {
  return Component.displayName || Component.name || 'MysteryComponent';
}

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var empty = {};

function createWithComponentState(_ref) {
  var prefix = _ref.prefix,
      reducers = _ref.reducers;

  return function withComponentState() {
    return function (WrappedComponent) {
      var DEFAULT_ID = 'default';

      var name = getDisplayName(WrappedComponent);

      var WithComponentState = function (_PureComponent) {
        _inherits(WithComponentState, _PureComponent);

        function WithComponentState(props) {
          _classCallCheck(this, WithComponentState);

          var _this = _possibleConstructorReturn(this, (WithComponentState.__proto__ || Object.getPrototypeOf(WithComponentState)).call(this, props));

          _this.setComponentState = _this.setComponentState.bind(_this);
          return _this;
        }

        _createClass(WithComponentState, [{
          key: 'componentDidUpdate',
          value: function componentDidUpdate(prevProps) {
            var prevId = prevProps.componentStateId;

            var _props = this.props,
                remove = _props.remove,
                componentStateId = _props.componentStateId;


            if (prevId === componentStateId) return;

            remove({
              name: name,
              id: this.id
            });
          }
        }, {
          key: 'componentWillUnmount',
          value: function componentWillUnmount() {
            this.props.remove({
              name: name,
              id: this.id
            });
          }
        }, {
          key: 'setComponentState',
          value: function setComponentState(state) {
            return this.props.setState({
              name: name,
              id: this.id,
              state: state
            });
          }
        }, {
          key: 'render',
          value: function render() {
            var componentState = this.props.componentState;


            var props = Object.assign({}, this.props);

            delete props.componentState;
            delete props.componentStateId;

            return React__default.createElement(WrappedComponent, _extends({}, props, {
              componentState: componentState[this.id] || empty,
              setState: this.setComponentState
            }));
          }
        }, {
          key: 'id',
          get: function get() {
            return this.props.componentStateId || DEFAULT_ID;
          }
        }]);

        return WithComponentState;
      }(React.PureComponent);

      WithComponentState.WrappedComponent = WrappedComponent;

      WithComponentState.displayName = 'WithComponentState(' + name + ')';

      function mapStateToProps(state) {
        var _state$prefix = state[prefix];
        _state$prefix = _state$prefix === undefined ? empty : _state$prefix;
        var _state$prefix$name = _state$prefix[name],
            componentState = _state$prefix$name === undefined ? empty : _state$prefix$name;


        return { componentState: componentState };
      }

      var mapDispatchToProps = createActions(prefix);

      return reactRedux.connect(mapStateToProps, mapDispatchToProps)(WithComponentState);
    };
  };
}

function create() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var defaults = {
    prefix: '@components',
    reducers: {}
  };

  var optionsWithDefaults = Object.assign({}, defaults, options);

  return {
    withComponentState: createWithComponentState(optionsWithDefaults),
    reducer: createReducer(optionsWithDefaults),
    prefix: optionsWithDefaults.prefix
  };
}

module.exports = create;
//# sourceMappingURL=index.js.map
