import React, {PureComponent} from 'react'
import {connect} from 'react-redux'

import createActions from './createActions'
import getDisplayName from './getDisplayName'

const empty = {}

export default function createWithComponentState ({prefix, reducers}) {
  return function withComponentState () {
    return WrappedComponent => {
      const DEFAULT_ID = 'default'

      const name = getDisplayName(WrappedComponent)

      class WithComponentState extends PureComponent {
        constructor (props) {
          super(props)
          this.setComponentState = this.setComponentState.bind(this)
        }

        get id () {
          return this.props.componentStateId || DEFAULT_ID
        }

        componentDidUpdate (prevProps) {
          const prevId = prevProps.componentStateId

          const {
            remove,
            componentStateId
          } = this.props

          if (prevId === componentStateId) return

          remove({
            name,
            id: this.id
          })
        }

        componentWillUnmount () {
          this.props.remove({
            name,
            id: this.id
          })
        }

        setComponentState (state) {
          return this.props.setState({
            name,
            id: this.id,
            state
          })
        }

        render () {
          const {
            componentState
          } = this.props

          const props = Object.assign({}, this.props)

          delete props.componentState
          delete props.componentStateId

          return (
            <WrappedComponent
              {...props}
              componentState={componentState[this.id] || empty}
              setState={this.setComponentState}
            />
          )
        }
      }

      WithComponentState.WrappedComponent = WrappedComponent

      WithComponentState.displayName = `WithComponentState(${name})`

      function mapStateToProps (state) {
        const {
          [prefix]: {
            [name]: componentState = empty
          } = empty
        } = state

        return {componentState}
      }

      const mapDispatchToProps = createActions(prefix)

      return connect(mapStateToProps, mapDispatchToProps)(WithComponentState)
    }
  }
}
